#!/usr/bin/env python3
# Python version of PDRK, written by Xin TAO (USTC), 2017-03-27
# Original Matlab version: Hua-sheng XIE (IFTS-ZJU), 2014
# http://hsxie.me/codes/pdrk/

import numpy as np
from numpy import sqrt,abs
from scipy.special import ive
from setbc import setbc

def read_paras(file):
    data = np.loadtxt(file, skiprows=1)

    if (data.ndim == 2):
        S, col = data.shape

        qs = data[:,0] # charge
        ms = data[:,1] # mass
        ns0 = data[:,2] # density
        Tzs =  data[:,3] # T parallel
        Tps =  data[:,4] # T perp
        vs0 = data[:,5]  # parallel drift velocity
    else:
        S = 1
        col = data.shape[0]

        qs = np.array([data[0]]) # charge
        ms = np.array([data[1]]) # mass
        ns0 = np.array([data[2]]) # density
        Tzs =  np.array([data[3]]) # T parallel
        Tps =  np.array([data[4]]) # T perp
        vs0 = np.array([data[5]])  # parallel drift velocity

    if col != 6:
        print('Wrong input data !!!')

    return {'S':S, 'qs':qs, 'ms':ms, 'ns0':ns0, 'Tzs':Tzs, 'Tps': Tps, 'vs0':vs0}

def em3d(file, k, N=2,J=8,B0=1, c2=1e4):
    kxv = k['kx']
    kzv = k['kz']

    paras = read_paras(file)  # can include any other parameters to the output by, for example, paras['c2']=c2

    qs = paras['qs']
    ms = paras['ms']
    ns0 = paras['ns0']
    Tzs = paras['Tzs']
    Tps = paras['Tps']
    vs0 = paras['vs0']
    S = paras['S']
    
    vtzs = np.sqrt(2*Tzs/ms)
    lambdaDs = np.sqrt(Tzs/(ns0*qs**2))
    kDs = 1/lambdaDs
    wps = np.sqrt(ns0*qs**2/ms)
    wcs = B0*qs/ms
    rhocs = sqrt(Tps/ms)/abs(wcs)

    wps2 = wps**2
    lmdT = Tzs/Tps

    c2 = 1e4

    epsilon0=1
    mu0=1/(c2*epsilon0);

    betasz=2*mu0*ns0*Tzs/B0**2
    betasp=2*mu0*ns0*Tps/B0**2
    vA=B0/np.sqrt(mu0*np.sum(ms*ns0)) 
    cS=np.sqrt(2*np.min(Tzs)/np.max(ms))

    bzj,czj=setbc(J)

    SNJ=S*(2*N+1)*J;
    SNJ1=SNJ+1;
    SNJ3=3*SNJ1;
    NN=SNJ3+6;

    sp=0;

    if (sp==0):
        nw=NN
    else:
        nw=10
        wg=1.5

    # -------------------------------------------
    kjv = np.arange(9.52, 0.01, -0.5)
    kjv = np.array([9.00])

    size_kxv = kxv.size
    size_kzv = kzv.size
    wmat = np.zeros((size_kxv, size_kzv, NN), dtype=np.complex)

    M = np.zeros((NN,NN), dtype=np.complex)
    csnj=np.zeros(3*SNJ, dtype=np.complex)
    b11snj = csnj*0
    b12snj = csnj*0
    b13snj = csnj*0
    b21snj = csnj*0
    b22snj = csnj*0
    b23snj = csnj*0
    b31snj = csnj*0
    b32snj = csnj*0
    b33snj = csnj*0

    for ikx, kx in enumerate(kxv):
        for ikz,kz in enumerate(kzv):        

            k = np.sqrt(kz**2+kx**2)
            bs = kx*rhocs
            bs[np.abs(bs)<1e-50] = 1e-50

            bs2 = bs**2
            M[:,:]=0
            snj = 0;

            b11 = 0
            b12 = 0
            b13 = 0
            b21 = 0
            b22 = 0
            b23 = 0
            b31 = 0
            b32 = 0
            b33 = 0

            csnj[:] = 0
            b11snj[:] = 0
            b12snj[:] = 0
            b13snj[:] = 0
            b21snj[:] = 0
            b22snj[:] = 0
            b23snj[:] = 0
            b31snj[:] = 0
            b32snj[:] = 0
            b33snj[:] = 0

            for s in np.arange(S):
                for n in np.arange(-N,N+1):

                    Gamn = ive(n,bs2[s])
                    Gamnp = (ive(n+1,bs2[s])+ive(n-1,bs2[s])-2*ive(n,bs2[s]))/2

                    for j in np.arange(J):
                        csnj[snj] = czj[j]*kz*vtzs[s]+kz*vs0[s]-n*wcs[s]

                        cnj = csnj[snj]
                        bj0 = vs0[s]+(1-1/lmdT[s])*czj[j]*vtzs[s]

                        if (n==0):
                            bnj1 = bj0/(czj[j]*vtzs[s]+vs0[s])
                        else:
                            bnj1=kz*bj0/cnj

                        bnj2 = 1-bnj1

                        tmp=wps2[s]*bzj[j]
                        b11snj[snj] = tmp*bnj2*n**2*Gamn/bs2[s]

                        b11=b11+tmp*bnj1*n**2*Gamn/bs2[s]

                        b12snj[snj]=tmp*bnj2*1j*n*Gamnp
                        b12=b12+tmp*bnj1*1j*n*Gamnp
                        b21snj[snj]=-b12snj[snj]
                        b21=-b12

                        b22snj[snj]=tmp*bnj2*(n**2*Gamn/bs2[s]-2*bs2[s]*Gamnp)
                        b22=b22+tmp*bnj1*(n**2*Gamn/bs2[s]-2*bs2[s]*Gamnp)

                        if(n==0):  # for eta_n*A_nj
                            bnj1=0 # avoid cnj=0 when kz=0
                        else:
                            bnj1=n*wcs[s]*bj0/cnj/vtzs[s]

                        bnj2=czj[j]/lmdT[s]-bnj1

                        b13snj[snj]=tmp*bnj2*n*sqrt(2*lmdT[s])*Gamn/bs[s]
                        b13=b13+tmp*bnj1*n*sqrt(2*lmdT[s])*Gamn/bs[s]
                        b31snj[snj]=b13snj[snj]
                        b31=b13

                        b23snj[snj]=-1j*tmp*bnj2*sqrt(2*lmdT[s])*Gamnp*bs[s]
                        b23=b23-1j*tmp*bnj1*sqrt(2*lmdT[s])*Gamnp*bs[s]
                        b32snj[snj]=-b23snj[snj]
                        b32=-b23;

                        if (bj0 == 0)+(kz==0):  # for eta_n^2*A_nj
                            bnj1=0
                            bnj2=czj[j]*czj[j]
                        else:
                            bnj1=n**2*bj0/cnj/vtzs[s]**2/kz
                            bnj2=(vs0[s]/vtzs[s]+czj[j])*czj[j]/lmdT[s]-n*wcs[s]*bj0*(1+n*wcs[s]/cnj)/vtzs[s]**2/kz;

                        b33snj[snj]=tmp*bnj2*2*lmdT[s]*Gamn
                        b33=b33+tmp*bnj1*2*lmdT[s]*Gamn

                        snj=snj+1

            for snj in range(SNJ):
                jjx=snj+0*SNJ1
                jjy=snj+1*SNJ1
                jjz=snj+2*SNJ1

                # v_snjx
                M[jjx,jjx] += csnj[snj]
                M[jjx,SNJ3] += b11snj[snj]
                M[jjx,SNJ3+1] += b12snj[snj]
                M[jjx,SNJ3+2] += b13snj[snj]

                # v_snjy
                M[jjy,jjy] += csnj[snj]
                M[jjy,SNJ3] += b21snj[snj]
                M[jjy,SNJ3+1] += b22snj[snj]
                M[jjy,SNJ3+2] += b23snj[snj]

                # v_snjz
                M[jjz,jjz] += csnj[snj]
                M[jjz,SNJ3] += b31snj[snj]
                M[jjz,SNJ3+1] += b32snj[snj]
                M[jjz,SNJ3+2] += b33snj[snj]

            tp=-1

            for jj in range(0*SNJ1, 1*SNJ1):
                ii=SNJ3
                M[ii,jj] += tp; 

            for jj in range(1*SNJ1, 2*SNJ1):
                ii=SNJ3+1
                M[ii,jj] += tp;

            for jj in range(2*SNJ1, 3*SNJ1):
                ii=SNJ3+2
                M[ii,jj] += tp

            # jx(E), jy(E), jz(E)
            M[1*SNJ1-1,SNJ3] += b11
            M[1*SNJ1-1,SNJ3+1] += b12
            M[1*SNJ1-1,SNJ3+2] += b13
            M[2*SNJ1-1,SNJ3] += b21
            M[2*SNJ1-1,SNJ3+1] += b22
            M[2*SNJ1-1,SNJ3+2] += b23
            M[3*SNJ1-1,SNJ3] += b31
            M[3*SNJ1-1,SNJ3+1] += b32
            M[3*SNJ1-1,SNJ3+2] += b33

            # E(B)
            M[SNJ3,SNJ3+4] += c2*kz
            M[SNJ3+1,SNJ3+3] += -c2*kz
            M[SNJ3+1,SNJ3+5] += c2*kx
            M[SNJ3+2,SNJ3+4] += -c2*kx

            # B(E)
            M[SNJ3+3,SNJ3+1] += -kz
            M[SNJ3+4,SNJ3] += kz
            M[SNJ3+4,SNJ3+2] += -kx
            M[SNJ3+5,SNJ3+1] += kx

            d,v = np.linalg.eig(M)

            omega = d
            sort_ind = np.argsort(omega.imag)[::-1]

            w = omega[sort_ind]

            wmat[ikx,ikz,:] = w

    return wmat, paras

if __name__ == '__main__':

    kzv = np.arange(9.52, 0.01, -0.5)/100
    kxv = np.array([0])

    wmat, paras = em3d('em3d.in', {'kx':kxv, 'kz':kzv})
    
    import matplotlib.pyplot as plt

    plt.close('all')

    plt.subplot(1,2,1)

    NW=20

    for i in range(NW):
#        plt.plot(kzv, wmat[0,:,i].real, 'C0*')
        plt.plot(kzv, wmat[0,:,i].real, 'c.')

    plt.subplot(1,2,2)
    for i in range(NW):
#        plt.plot(kzv, wmat[0,:,i].imag, 'C0*')
        plt.plot(kzv, wmat[0,:,i].imag, 'c.')

    plt.show()
