#!/usr/bin/env python3
#Python version of PDRK, written by Xin TAO (USTC), 2017-03-27
# Original Matlab version: Hua-sheng XIE (IFTS-ZJU), 2014
# http://hsxie.me/codes/pdrk/

import numpy as np
from setbc import setbc
from scipy.special import ive
import matplotlib.pyplot as plt

def read_paras(file):
    data = np.loadtxt(file, skiprows=1)

    if (data.ndim == 2):
        S, col = data.shape

        qs = data[:,0] # charge
        ms = data[:,1] # mass
        ns0 = data[:,2] # density
        Tzs =  data[:,3] # T parallel
        Tps =  data[:,4] # T perp
        vs0 = data[:,5]  # parallel drift velocity
    else:
        S = 1
        col = data.shape[0]

        qs = np.array([data[0]]) # charge
        ms = np.array([data[1]]) # mass
        ns0 = np.array([data[2]]) # density
        Tzs =  np.array([data[3]]) # T parallel
        Tps =  np.array([data[4]]) # T perp
        vs0 = np.array([data[5]])  # parallel drift velocity

    if col != 6:
        print('Wrong input data !!!')

    return {'S':S, 'qs':qs, 'ms':ms, 'ns0':ns0, 'Tzs':Tzs, 'Tps': Tps, 'vs0':vs0}

def es3d(file, k, N=10, J=8, B0=1):
    kxv = k['kx']
    kzv = k['kz']

    paras = read_paras(file)
    
    qs = paras['qs']
    ms = paras['ms']
    ns0 = paras['ns0']
    Tzs = paras['Tzs']
    Tps = paras['Tps']
    vs0 = paras['vs0']
    S = paras['S']

    vtzs=np.sqrt(2*Tzs/ms) # para thermal velocity
    lambdaDs=np.sqrt(Tzs/(ns0*qs**2)) # Debye length, Tzs
    kDs=1/lambdaDs 
    wps=np.sqrt(ns0*qs**2/ms) # plasma frequency
    wcs=B0*qs/ms              # cyclotron frequency
    rhocs=np.sqrt(Tps/ms)/np.abs(wcs) # cyclotron radius

    bzj,czj=setbc(J)

    paras['wps'] = wps # this is included in the output paras to be used in test()
    paras['wcs'] = wcs # this is included in the output paras to be used in test()

    SNJ=S*(2*N+1)*J

    paras['SNJ'] = SNJ # this is included in the output paras to be used in test()

    M=np.zeros((SNJ,SNJ), dtype=np.complex);
    csnj = np.zeros(SNJ, dtype=np.complex)
    bsnj = np.zeros(SNJ, dtype=np.complex)

    size_kxv = kxv.size
    size_kzv = kzv.size
    wmat = np.zeros((size_kxv, size_kzv, SNJ), dtype=np.complex)

    for ikx, kx in enumerate(kxv):
        for ikz,kz in enumerate(kzv):

            k=np.sqrt(kz**2+kx**2);
            bs=kx*rhocs;
            bs2=bs**2;

            M[:,:]=0
            bsnj[:]=0
            csnj[:]=0

            snj=0
            aSNJ=1

            for s in range(S):
                aSNJ=aSNJ+(kDs[s]**2/k**2)

                for n in np.arange(-N, N+1, 1):
                    Gamn=ive(n,bs2[s])

                    for j in range(J):
                        tmp=n*wcs[s]+czj[j]*kz*vtzs[s]
                        csnj[snj]=kz*vs0[s]+tmp
                        bsnj[snj]=(kDs[s]**2/k**2)*Gamn*bzj[j]*(tmp-(1-Tzs[s]/Tps[s])*n*wcs[s])
                        aSNJ=aSNJ+(kDs[s]**2/k**2)*Gamn*(bzj[j])
                        snj=snj+1

            for snj in range(SNJ):
                M[snj,:]=-bsnj[snj]
                M[snj,snj]=-bsnj[snj]+aSNJ*csnj[snj]

            omega,v=np.linalg.eig(M)

            sort_ind = np.argsort(omega.imag)[::-1]
            w = omega[sort_ind]

            wmat[ikx,ikz,:] = w

    return wmat, paras

def test():
    kx = np.arange(0.05, 5, 0.1)
    kz = np.array([0.0])

    k = {'kx':kx, 'kz':kz}

    file = 'es3d.in'
    wmat,paras = es3d(file, k)

    wmat = np.squeeze(wmat)

    plt.close()

    fig, axes = plt.subplots(1,2)

    ax=axes[0]

    for n in range(6):
#        ax.plot([0,kx.max()], [n*np.abs(paras['wcs'][0]), n*np.abs(paras['wcs'][0])], '--', color='C0')
        ax.plot([0,kx.max()], [n*np.abs(paras['wcs'][0]), n*np.abs(paras['wcs'][0])], '--', color='c')

    for j in range(paras['SNJ']):
#        ax.plot(kx, wmat[:,j].real, 'C0*')
        ax.plot(kx, wmat[:,j].real, 'c.')

    ax.set_xlim([0,np.max(kx)])
    ax.set_ylim(0, 5.5)

    ax.set_title(r'$\omega_p/\omega_c=$'+str(np.abs(paras['wps'][0]/paras['wcs'][0])))
    ax.set_xlabel(r'$k_\perp/\rho_c$')
    ax.set_ylabel(r'$\omega_r/\omega_c$')

    ax = axes[1]

    for j in range(paras['SNJ']):
        ax.plot(kx,wmat[:,j].imag, 'c.')
#        ax.plot(kx,wmat[:,j].imag, 'C0*')

    ax.set_xlim([0,kx.max()])
    ax.set_xlabel(r'$k_\perp/\rho_c$')
    ax.set_ylabel(r'$\omega_i/\omega_c$')
        
    plt.show()

if __name__ == '__main__':

    test()

